import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis,
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatting the total value to show on the Areachart
const xAxisTickFormat = (number) => format('.3s')(number);

const render = (data) => {
  const xValue = (d) => d.year;
  const xAxisLabel = 'year';

  const yValue = (d) => d.Nuclear;
  const circleRadius = 7;
  const yAxisLabel = 'Nuclear usage';

  const margin = {
    top: 12,
    right: 43,
    bottom: 80,
    left: 140,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  // finding the Scale for X axis
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  // finding the Scale for Y axis
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  // placing the svg element
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );
  // placing values to x axis
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(17);

  // placing values to y axis
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(11)
    .tickFormat(xAxisTickFormat);

  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -91)
    .attr('x', -innerHeight / 2)
    .attr('transform', `rotate(-91)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 75)
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);
  // To draw the line on the curve and adding it to svg
  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d)))
    .curve(curveBasis);
  // To draw the area under the curve and adding it to svg
  const areaGenerator = area()
    .x((d) => xScale(xValue(d)))
    .y0(innerHeight)
    .y1((d) => yScale(yValue(d)))
    .curve(curveBasis);

  g.append('path')
    .attr('class', 'line-chart')
    .attr('d', lineGenerator(data));

  g.append('path')
    .attr('class', 'area-chart')
    .attr('d', areaGenerator(data));
};
// Loading and Parsing CSV data
csv(
  'https://gist.githubusercontent.com/sriharsha729/092085da6c443c1046c8bc09e42deec2/raw/c68a9e46e0838fcbde5a1eb526638f947b978dd9/nucleardata.csv',
).then((data) => {
  data.forEach((d) => {
    d.Nuclear = +d.Nuclear;
    //Formatting to retrieve year from the date
    d.year = d3.timeParse('%Y-%m-%d')(d.year);
  });
  render(data);
  console.log(data);
});
